<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8"/>
    <meta http-equiv="X-UA-Compatible" content="IE=edge"/>
    <meta name="viewport" content="width=device-width, initial-scale=1.0"/>
    <title>Markmap</title>
    <style>
        svg.markmap {
            width: 100%;
            height: 100vh;
        }
    </style>
    <script src="https://cdn.jsdelivr.net/npm/markmap-autoloader"></script>
</head>
<body>
<div class="markmap">
    <script type="text/template">

        {{md}}

    </script>
</div>
</body>
</html>
